@@ -18,11 +18,8 @@ test_that("ggR returns proper ggplot2 classes or data.frames", {
                 for(s in which(with(tests, ggObj & ggLayer & !anno))) expect_equal(builds[[s]]$geom$objname, "raster", info = tinfo[s])                       
             } else {
                 ## Upcoming ggplot2 version (>=1.0.1.9002)
-                for(s in which(with(tests, ggObj & ggLayer))) expect_is(builds[[s]]$geom, "GeomRaster", info = tinfo[s])
                 for(s in which(with(tests, ggObj & ggLayer & anno)))  expect_is(builds[[s]]$geom, "GeomRasterAnn", info = tinfo[s])
-                for(s in which(with(tests, ggObj & ggLayer & !anno))) expect_is(builds[[s]]$geom, "GeomRaster", info = tinfo[s])                       
-                
-                
+                for(s in which(with(tests, ggObj & ggLayer & !anno))) expect_is(builds[[s]]$geom, "GeomRaster", info = tinfo[s])                                      
             }
             ## Data.frames
             for(s in  which(with(tests, !ggObj))) expect_is(builds[[s]], "data.frame", info = tinfo[s])
@@ -52,7 +49,6 @@ test_that("ggRGB returns proper ggplot2 classes or data.frames", {
                 for(s in which(with(tests, ggObj & ggLayer & !anno))) expect_equal(builds[[s]]$geom$objname, "raster", info = tinfo[s])                       
             } else {
                 ## Upcoming ggplot2 version (>=1.0.1.9002)
-                for(s in which(with(tests, ggObj & ggLayer))) expect_is(builds[[s]]$geom, "GeomRaster", info = tinfo[s])
                 for(s in which(with(tests, ggObj & ggLayer & anno)))  expect_is(builds[[s]]$geom, "GeomRasterAnn", info = tinfo[s])
                 for(s in which(with(tests, ggObj & ggLayer & !anno))) expect_is(builds[[s]]$geom, "GeomRaster", info = tinfo[s])                       
             }    
